
import asyncio
import time
import logging

from collections import deque

logger = logging.getLogger('aionasa.rate_limit')



class RateLimiter:
    """
    Class that handles rate limit across the whole library.
    This is necessary to ensure observation of rate limits even if multiple API endpoints are being used.
    """

    def __init__(self, limit):
        logger.debug("Initializing `RateLimiter`")
        self._requests = deque(iterable=[time.monotonic()]*limit, maxlen=limit)
        self._limit = limit
        self._remaining = limit

    @property
    def remaining(self):
        return self._remaining

    async def wait(self):
        logger.debug("`RateLimiter.wait()` was called.")
        if self._remaining < 1:
            timestamp = self._requests.popleft()
            time_to_wait = 3600 - (time.monotonic() - timestamp)
            if time_to_wait > 0:
                logger.debug(f"Sleeping for {time_to_wait} seconds.")
                await asyncio.sleep(time_to_wait)
            else:
                logger.debug(f"Skipping sleep due to invalid sleep duration: {time_to_wait}")
        else:
            logger.debug(">0 requests remaining, sleep is not necessary.")

    def update(self, remaining):
        logger.debug(f"Updating rate limit: {remaining} requests remaining.")
        self._remaining = remaining
        self._requests.append(time.monotonic())


default_rate_limiter = RateLimiter(1000)
# default_rate_limiter.__repr__ = lambda: '<default_rate_limiter>'
insight_rate_limiter = RateLimiter(2000)
# insight_rate_limiter.__repr__ = lambda: '<insight_rate_limiter>'
demo_rate_limiter = RateLimiter(30)  # todo: figure out how to implement daily limit
# demo_rate_limiter.__repr__ = lambda: '<demo_rate_limiter>'


# class RateLimiter:
#     def __init__(self):
#         self._available_requests = 1000
#
#     async def _restore(self):
#         await asyncio.sleep(3600)
#         self._available_requests += 1
#
#     async def _wait(self):
#         while self._available_requests < 1:
#             await asyncio.sleep(10**-6)
#
#     async def update(self):
#         if self._available_requests < 1:
#             await self._wait()
#         else:
#             self._available_requests -= 1
#             asyncio.create_task(self._restore())


# class RateLimiter2:
#     def __init__(self):
#         self._requests = []
#         self._count = 0
#
#     async def update(self):
#         if self._count > 999:
#             timestamp = self._requests.pop(0)
#             await asyncio.sleep(time.monotonic() - timestamp)
#             self._count -= 1
#         else:
#             self._requests.append(time.monotonic())
#             self._count += 1



import aiohttp
import asyncio
import argparse

import json
import yaml

from datetime import date, timedelta, datetime

from .api import APOD
from ..errors import ArgumentError


__doc__ = """

CLI TOOL FOR APOD API WRAPPER

"""

VALID_FILE_TYPES = ['csv', 'json', 'yaml']



parser = argparse.ArgumentParser(description="CLI tool for APOD API wrapper.")

parser.add_argument('--date', '-d', help="Request APOD data for a single date.")
parser.add_argument('--start-date', '--from', help="The first date to return when requesting a range of dates.")
parser.add_argument('--end-date', '--to', help="The last date to return when requesting a range of dates. Range is inclusive.")
parser.add_argument('--since', help="Shorthand that selects date range up to the current date")
parser.add_argument('--print', '-p', action='store_true', help="Flag indicating that data should be printed in a human-readable format In addition to other actions.")
parser.add_argument('--dump', help="Indicates that data should be dumped to a file. Supports json and yaml/yml extensions.")
parser.add_argument('--download', help="After data is retrieved, downloads images to the given directory.")
parser.add_argument('--key', help="Manual input option for API key. If this is left out, uses DEMO_KEY.")
parser.add_argument('--timeout', type=int, help="Configures timeout settings for the underlying aiohttp.ClientSession. Overrides the 'total' attribute.")


def process_date(day):
    keywords = {
        'today': date.today(),
        'yesterday': date.today() - timedelta(days=1)
    }
    if day in keywords.keys():
        return keywords[day]

    return datetime.strptime(day, f'%Y-%m-%d').date()


def dump_to_json(data, filename):
    with open(filename, 'w') as f:
        json.dump(data, f)


def dump_to_csv(data, filename):
    with open(filename, 'w') as f:
        if not isinstance(data, list):  # batch_get
            data = [data]
        for _dict in data:
            items = [str(value) for value in _dict.values()]
            f.write(','.join(items) + '\n')


def dump_to_yaml(data, filename):
    with open(filename, 'w') as f:
        yaml.dump(data, f, yaml.Dumper)


async def get(day, _print, dump, download, key, timeout):
    day = process_date(day)
    if timeout is not None:
        timeout = aiohttp.ClientTimeout(total=timeout)
    else:
        timeout = aiohttp.ClientTimeout()
        
    async with APOD(key or 'DEMO_KEY', timeout=timeout) as apod:
        picture = await apod.get(day)
        data = picture.json()

        if _print:
            s = ""
            for key, value in data.items():
                s += f"{key}: {value}\n"
            print(s)

        if dump:
            split = dump.split('.')
            ext = split[-1]

            if ext.lower() == 'json':
                dump_to_json(data, dump)
            elif ext.lower() == 'csv':
                dump_to_csv(data, dump)
            elif ext.lower() == 'yaml':
                dump_to_yaml(data, dump)

        if download:
            await download_image(picture, download)


async def batch_get(start_date, end_date, _print, dump, download, key, timeout):
    start_date = process_date(start_date)
    end_date = process_date(end_date)
    if timeout is not None:
        timeout = aiohttp.ClientTimeout(total=timeout)
    else:
        timeout = aiohttp.ClientTimeout()
    
    async with APOD(key or 'DEMO_KEY', timeout=timeout) as apod:
        data = await apod.batch_get(start_date, end_date)
        json_data = [picture.json() for picture in data]

        if _print:
            s = ""
            for entry in json_data:
                for key, value in entry.items():
                    s += f"{key}: {value}\n"
                s += "\n"
            print(s)

        if dump:
            split = dump.split('.')
            ext = split[-1]

            if ext.lower() == 'json':
                dump_to_json(data, dump)
            elif ext.lower() == 'csv':
                dump_to_csv(data, dump)
            elif ext.lower() in {'yaml', 'yml'}:
                dump_to_yaml(data, dump)

        if download:
            await asyncio.gather(*[download_image(picture, download, True) for picture in data])


async def download_image(picture, path, ignore_not_implemented=False):
    url = picture.hdurl if picture.hdurl else picture.url

    if url.startswith('http://apod.nasa.gov') or url.startswith('https://apod.nasa.gov'):
        filename = url.split('/')[-1]

    else:  # todo: add support for non-apod images (youtube etc)
        if ignore_not_implemented:
            print(f"Could not download url, skipping: {url}")
            return 0
        raise NotImplementedError("URL not supported by download function. File could not be downloaded.")

    filepath = path + '/' + filename
    try:
        print(f"Starting download: {url}")
        b = await picture.save(filepath)
        print(f"Downloaded: {url} ({b/1000} KB)")
    except asyncio.TimeoutError:
        print(f"Download failed: {url}")
        raise


async def main():
    args = parser.parse_args()
    _date = args.date
    start_date = args.start_date
    end_date = args.end_date
    since = args.since
    _print = args.print
    dump = args.dump
    download = args.download
    key = args.key
    timeout = args.timeout

    if _date:
        if start_date or end_date or since:
            raise ArgumentError("date and start-date/end-date/since are not compatible arguments.")
        await get(_date, _print, dump, download, key, timeout)

    elif since:
        if start_date or end_date:
            raise ArgumentError("since and start-date/end-date are not compatible arguments.")
        await batch_get(since, 'today', _print, dump, download, key, timeout)

    elif start_date or end_date:
        if not (start_date and end_date):
            raise ArgumentError("start-date and end-date are both required arguments when requesting a range.")
        await batch_get(start_date, end_date, _print, dump, download, key, timeout)

    else:
        await get('today', _print, dump, download, key, timeout)



if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())




import datetime
import logging
from typing import List

from aiohttp import ClientSession

from ..client import BaseClient
from ..errors import *
from ..rate_limit import default_rate_limiter, demo_rate_limiter, RateLimiter
from .data import AstronomyPicture


logger = logging.getLogger('aionasa.apod')


class _DefaultRateLimiterRepr:
    def __repr__(self):
        return '<default_rate_limiter>'


_default = _DefaultRateLimiterRepr()


class APOD(BaseClient):
    """Client for NASA Astronomy Picture of the Day API.

    Parameters
    ----------
    api_key: :class:`str`
        NASA API key to be used by the client.
    session: Optional[:class:`aiohttp.ClientSession`]
        Optional ClientSession to be used for requests made by this client. Creates a new session by default.
    rate_limiter: Optional[:class:`RateLimiter`]
        Optional RateLimiter class to be used by this client. Uses the library's internal global rate limiting by default.
    """

    def __init__(self, api_key='DEMO_KEY', session=None, rate_limiter=_default, **kwargs):
        if api_key == 'DEMO_KEY' and rate_limiter:
            rate_limiter = demo_rate_limiter
        elif rate_limiter is _default:
            rate_limiter = default_rate_limiter
        super().__init__(api_key, session, rate_limiter, **kwargs)

    async def get(self, date: datetime.date = None, hd: bool = False, as_json: bool = False):
        """Retrieves a single item from NASA's APOD API.

        Parameters
        ----------
        date: :class:`datetime.Date`
            The date of the APOD image to retrieve. Defaults to ``'today'``.
        hd: :class:`bool`
            Bool indicating whether to retrieve the URL for the high resolution image. Defaults to ``False``.
        as_json: :class:`bool`
            Bool indicating whether to return the raw returned json data instead of the normal AstronomyPicture object. Defaults to ``False``.

        Returns
        -------
        :class:`AstronomyPicture`
            An AstronomyPicture containing data returned by the API.
        """

        if date is None:  # parameter will be left out of the query.
            date = ''
        else:
            date = 'date=' + date.strftime('%Y-%m-%d') + '&'
        if hd is None:  # parameter will be left out of the query.
            hd = ''
        else:
            hd = 'hd=' + str(hd) + '&'

        request = f"https://api.nasa.gov/planetary/apod?{date}{hd}api_key={self._api_key}"

        if self.rate_limiter:
            await self.rate_limiter.wait()

        async with self._session.get(request) as response:
            if response.status != 200:  # not success
                raise APIException(response.status, response.reason)

            json = await response.json()

        if self.rate_limiter:
            remaining = int(response.headers['X-RateLimit-Remaining'])
            self.rate_limiter.update(remaining)

        if as_json:
            return json

        else:
            date = json.get('date')
            date = datetime.datetime.strptime(date, '%Y-%m-%d').date() if date else None

            entry = AstronomyPicture(
                client=self,
                date=date,
                copyright=json.get('copyright'),
                title=json.get('title'),
                explanation=json.get('explanation'),
                url=json.get('url'),
                hdurl=json.get('hdurl'),
                media_type=json.get('media_type'),
                service_version=json.get('service_version'),
            )
            return entry

    async def batch_get(self, start_date: datetime.date, end_date: datetime.date,
                        hd: bool = None, as_json: bool = False):
        """Retrieves multiple items from NASA's APOD API. Returns a list of APOD entries.

        Parameters
        ----------
        start_date: :class:`datetime.Date`
            The first date to return when requesting a range of dates.
        end_date: :class:`datetime.Date`
            The last date to return when requesting a range of dates. Range is inclusive.
        hd: :class:`bool`
            Bool indicating whether to retrieve the URL for the high resolution image. Defaults to ``False``.
        as_json: :class:`bool`
            Bool indicating whether to return a list of dicts containing the raw returned json data instead of the normal ``List[AstronomyPicture]``. Defaults to ``False``.

        Returns
        -------
        List[:class:`AstronomyPicture`]
            A list of AstronomyPicture objects containing data returned by the API.
        """

        start_date = 'start_date=' + start_date.strftime('%Y-%m-%d') + '&'
        end_date = 'end_date=' + end_date.strftime('%Y-%m-%d') + '&'

        if hd is None:  # parameter will be left out of the query.
            hd = ''
        else:
            hd = 'hd=' + str(hd) + '&'

        request = f"https://api.nasa.gov/planetary/apod?{start_date}{end_date}{hd}api_key={self._api_key}"

        if self.rate_limiter:
            await self.rate_limiter.wait()

        async with self._session.get(request) as response:
            if response.status != 200:  # not success
                raise APIException(response.status, response.reason)

            json = await response.json()

        if self.rate_limiter:
            remaining = int(response.headers['X-RateLimit-Remaining'])
            self.rate_limiter.update(remaining)

        if as_json:
            return json

        else:
            result = []

            for item in json:

                date = item.get('date')
                date = datetime.datetime.strptime(date, '%Y-%m-%d').date() if date else None

                entry = AstronomyPicture(
                    client=self,
                    date=date,
                    copyright=item.get('copyright'),
                    title=item.get('title'),
                    explanation=item.get('explanation'),
                    url=item.get('url'),
                    hdurl=item.get('hdurl'),
                    media_type=item.get('media_type'),
                    service_version=item.get('service_version')
                )
                result.append(entry)

            return result






from ..errors import *

"""
To anyone reading this:
    I'm sorry.
"""


class _QueryString:
    def __init__(self, string):
        self.str = string

    def __bool__(self):
        return NotImplemented

    def __str__(self):
        return self.str

    def __repr__(self):
        return f"QueryString('{self}')"

    def __and__(self, other):
        return _QueryString(f"{self.str} and {other.str}")

    def __or__(self, other):
        return _QueryString(f"{self.str} or {other.str}")


class _Query:
    def __init__(self, name):
        self.name = name

    def __bool__(self):
        return NotImplemented

    def __str__(self):
        return self.name

    def __repr__(self):
        return f"Query('{self}')"

    def __eq__(self, other):
        return _QueryString(f"{self.name}={other}")

    def __gt__(self, other):
        return _QueryString(f"{self.name}>{other}")

    def __lt__(self, other):
        return _QueryString(f"{self.name}<{other}")

    def __ge__(self, other):
        return _QueryString(f"{self.name}>={other}")

    def __le__(self, other):
        return _QueryString(f"{self.name}<={other}")

    def like(self, item):
        return _QueryString(f"{self.name} like {item}")

    def __contains__(self, item):
        return _QueryString(f"{self.name} like {item}")


"""
PUT QUERY OPTIONS HERE

ex:
    ra = _Query('ra')
    dec = _Query('dec')
    pl_discmethod = _Query('pl_discmethod')
"""


# =====================================================================================================================================================

class _Format:
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name


csv = _Format('csv')
bar = _Format('bar')
bar_delimited = _Format('bar_delimited')
pipe = _Format('pipe')
pipe_delimited = _Format('pipe_delimited')
xml = _Format('xml')
vo_table = _Format('vo_table')
json = _Format('json')


# =====================================================================================================================================================

class _Table:  # my god... this is going to be an ugly one.
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name







"""
aio-nasa exceptions
"""


class NASAException(Exception):
    pass


class APIException(NASAException):
    def __init__(self, code, reason):
        self.code = code
        self.reason = reason
        super().__init__(f"{code} - {reason}")


class ArgumentError(NASAException):
    pass


class PandasNotFound(NASAException):
    pass


# class NotFound(APIException):
#     pass
#
#
# class TooManyRequests(APIException):
#     pass




import asyncio
import json

from .api import InSight


async def main():
    async with InSight() as insight:
        data = await insight.get()
        print(data)
        # print(json.dumps(data, sort_keys=True, indent=4))



if __name__ == '__main__':
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())





import datetime

from ..errors import APIException, NASAException


class AstronomyPicture:
    """A class representing a single daily APOD picture.

    Attributes
    ----------
    client: :class:`APOD`
        The APOD client that was used to retrieve this data.
    date: :class:`datetime.Date`
        The date this image was uploaded to APOD.
    copyright:
        The owner of this image, if it is not public domain.
    title:
        The APOD entry's title.
    explanation:
        The explanation for this image, written by astronomers, from this image's APOD page.
    url:
        The image url.
    hdurl:
        The HD image url, if available. Can be ``None``.
    html_url:
        The url of the APOD HTML page. This is the page a user would find this image on.
        This data is not provided by the API. This attribute has been added by the library for ease of use.
    media_type:
        The type of media. Will pretty much always be ``'image'``.
    service_version:
        The API service version. The API version is currently ``'v1'``.
    """
    def __init__(self, client, date: datetime.date, **kwargs):
        self.client = client
        self.date = date
        self.copyright = kwargs.get('copyright')
        self.title = kwargs.get('title')
        self.explanation = kwargs.get('explanation')
        self.url = kwargs.get('url')
        self.hdurl = kwargs.get('hdurl')
        self.media_type = kwargs.get('media_type')
        self.service_version = kwargs.get('service_version')

        site_formatted_date = f"{str(date.year)[2:]}{date.month:02d}{date.day:02d}"
        self.html_url = f"https://apod.nasa.gov/apod/ap{site_formatted_date}.html"

        self._response = None

    def json(self):
        """Convert this object to JSON format.

        Returns
        -------
        :class:`dict`
            The JSON data that was provided by the APOD API.
        """
        return {
            'date': self.date.strftime('%Y-%m-%d'),
            'copyright': self.copyright,
            'title': self.title,
            'explanation': self.explanation,
            'url': self.url,
            'hdurl': self.hdurl,
            'html_url': self.html_url,
            'media_type': self.media_type,
            'service_version': self.service_version,
        }

    async def read(self, hdurl: bool = True):
        """Downloads the image associated with this AstronomyPicture.

        Parameters
        ----------
        hdurl: :class:`bool`
            Indicates that the HD image should be downloaded, if possible.

        Returns
        -------
        :class:`bytes`
            The image, downloaded from the URL provided by the API.
        """

        if hdurl and self.hdurl:
            url = self.hdurl
        else:
            url = self.url

        if not (url.startswith('http://apod.nasa.gov') or url.startswith('https://apod.nasa.gov')):
            raise NotImplementedError("URLs from outside apod.nasa.gov are not currently supported.")

        async with self.client._session.get(url) as response:
            if response.status != 200:
                raise APIException(response.status, response.reason)
            image = await response.read()

        return image

    async def save(self, path=None, hdurl: bool = True):
        """Downloads the image associated with this AstronomyPicture and saves to a file.

        Parameters
        ----------
        path:
            The file path at which to save the image.
            If ``None``, saves the image to the working directory using the filename from the image url.
        hdurl: :class:`bool`
            Indicates that the HD image should be downloaded, if possible.
        """

        if hdurl and self.hdurl:
            url = self.hdurl
        else:
            url = self.url

        path = path if path else f"./{url.split('/')[-1]}"

        if not (url.startswith('http://apod.nasa.gov') or url.startswith('https://apod.nasa.gov')):
            raise NotImplementedError("URLs from outside apod.nasa.gov are not currently supported.")

        async with self.client._session.get(url) as response:
            if response.status != 200:
                raise APIException(response.status, response.reason)
            with open(path, 'wb') as f:
                bytes_written = 0
                while True:
                    chunk = await response.content.read(1000)
                    bytes_written += len(chunk)
                    if not chunk:
                        break
                    f.write(chunk)

        return bytes_written

    # def stream(self, hdurl: bool = True):
    #     if hdurl and self.hdurl:
    #         url = self.hdurl
    #     else:
    #         url = self.url
    #
    #     if not (url.startswith('http://apod.nasa.gov') or url.startswith('https://apod.nasa.gov')):
    #         raise NotImplementedError("URLs from outside apod.nasa.gov are not currently supported.")
    #
    #     return _StreamReader(self, self.client._session.get(url))


    async def read_chunk(self, chunk_size: int, hdurl: bool = True):
        """Reads a chunk of the image associated with this AstronomyPicture.

        Parameters
        ----------
        chunk_size: :class:`int`
            Number of bytes to read.
        hdurl: :class:`bool`
            Indicates that the HD image should be downloaded, if possible.

        Returns
        -------
        :class:`bytes`
            The chunked data. Will be None if the image has been completely read.
        """

        if hdurl and self.hdurl:
            url = self.hdurl
        else:
            url = self.url

        if not (url.startswith('http://apod.nasa.gov') or url.startswith('https://apod.nasa.gov')):
            raise NotImplementedError("URLs from outside apod.nasa.gov are not currently supported.")

        if not self._response:
            response = await self.client._session.get(url)
            if response.status != 200:
                await response.close()
                raise APIException(response.status, response.reason)
            self._response = response

        chunk = await self._response.content.read(chunk_size)
        if not chunk:
            await self._response.close()
            self._response = None
        return chunk


# class _StreamReader:
#     def __init__(self, picture, response):
#         self.picture = picture
#         self.response = response
#
#     async def __aenter__(self):
#         await self.response.__aenter__()
#         if self.response.status != 200:
#             raise APIException(self.response.status, self.response.reason)
#         return self
#
#     async def __aexit__(self, exc_type, exc_val, exc_tb):
#         return await self.response.__aexit__(exc_type, exc_val, exc_tb)
#
#     async def read(self, num):



import aiohttp
import datetime
import logging

from ..client import BaseClient
from ..errors import *
from ..rate_limit import default_rate_limiter, demo_rate_limiter


logger = logging.getLogger('aionasa.apod')


class NeoWs(BaseClient):
    """
    Client for NASA Astronomy Picture of the Day.

    Parameters for NeoWs API:
        -
    """

    def __init__(self, api_key='DEMO_KEY', session=None, rate_limiter=default_rate_limiter):
        if api_key == 'DEMO_KEY' and rate_limiter:
            rate_limiter = demo_rate_limiter
        super().__init__(api_key, session, rate_limiter)

    async def feed(self, start_date: datetime.date, end_date: datetime.date = None):
        """
        Retrieve a list of Asteroids based on their closest approach date to Earth.

        :param start_date: Starting date for asteroid search.
        :param end_date: Ending date for asteroid search.
        :return: A list of Asteroids returned by the API.
        """

        start_date = 'start_date=' + start_date.strftime('%Y-%m-%d') + '&'

        if end_date is None:  # parameter will be left out of the query.
            end_date = ''
        else:
            end_date = 'end_date=' + end_date.strftime('%Y-%m-%d') + '&'

        request = f"https://api.nasa.gov/neo/rest/v1/feed?{start_date}{end_date}api_key={self._api_key}"

        if self.rate_limiter:
            await self.rate_limiter.wait()

        async with self._session.get(request) as response:
            if response.status != 200:  # not success
                raise APIException(response.status, response.reason)

            json = await response.json()

        if self.rate_limiter:
            remaining = int(response.headers['X-RateLimit-Remaining'])
            self.rate_limiter.update(remaining)







import aiohttp


class BaseClient:
    """
    Base class for NASA API clients.
    """

    def __init__(self, api_key='DEMO_KEY', session=None, rate_limiter=None, timeout=None):
        """
        Initializes the client class.

        :param api_key: api.nasa.gov key for expanded usage.
        :param session:
        """
        self._api_key = api_key
        timeout = timeout or aiohttp.ClientTimeout()
        self._session = session or aiohttp.ClientSession(timeout=timeout)
        self.rate_limiter = rate_limiter

    # def __call__(self, *args, **kwargs):
    #     return self.get(*args, **kwargs)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self._session:
            await self._session.close()

    async def close(self):
        if self._session:
            await self._session.close()





import setuptools

with open("./README.md", "r") as fh:
    long_description = fh.read()

with open('./aionasa/__init__.py', 'r') as f:
    # FIRST LINE:
    # __version__ = '<version>'
    line = f.readline()
    version = eval(line[14:])

setuptools.setup(
    name="aionasa",
    version=version,
    author="nwunderly",
    author_email="",
    description="An async python wrapper for NASA open APIs.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nwunderly/aio-nasa",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)




import aiohttp
import asyncio
import datetime
import logging

try:
    import pandas
except ImportError:
    pandas = None

from collections import namedtuple

from ..client import BaseClient
from ..errors import *
from ..rate_limit import insight_rate_limiter, demo_rate_limiter


logger = logging.getLogger('aionasa.insight')


class InSight(BaseClient):
    """
    Client for NASA Insight Mars weather data.

    In-depth documentation for this API can be found at https://api.nasa.gov/assets/insight/InSight%20Weather%20API%20Documentation.pdf

    Parameters for InSight API:
        - version: The version of this API. Locked to 1.0 currently.
        - feedtype: The format of what is returned. Currently the default is JSON and only JSON works.
    """

    def __init__(self, api_key='DEMO_KEY', session=None, rate_limiter=insight_rate_limiter):
        if api_key == 'DEMO_KEY' and rate_limiter:
            rate_limiter = demo_rate_limiter
        super().__init__(api_key, session, rate_limiter)

    async def get(self, feedtype='json', as_json=False):
        """
        Retrieves Mars weather data from the last seven available days.

        :param feedtype: The format of what is returned. Currently the default is JSON and only JSON works.
        :param as_json: Bool indicating whether to return a dict containing the raw returned json data instead of the normal named tuple.
        :return: A pandas DataFrame containing data returned by the API.
        """

        request = f"https://api.nasa.gov/insight_weather/?ver=1.0&feedtype={feedtype}&api_key={self._api_key}"

        if self.rate_limiter:
            await self.rate_limiter.wait()

        async with self._session.get(request) as response:
            if response.status != 200:  # not a success
                raise APIException(response.status, response.reason)

            json = await response.json()

        if self.rate_limiter:
            remaining = int(response.headers['X-RateLimit-Remaining'])
            self.rate_limiter.update(remaining)

        if as_json:
            return json

        else:

            if not pandas:
                raise PandasNotFound("Package 'pandas' could not be imported. DataFrame data format cannot be used.")

            sol_keys = json.pop('sol_keys')
            validity_checks = json.pop('validity_checks')
            data = {'sol': [], 'AT': [], 'HWS': [], 'PRE': [], 'WD': [], 'Season': [], 'First_UTC': [], 'Last_UTC': [], 'validity_checks': []}
            for sol, sol_data in json.items():
                data['sol'].append(sol)
                for col_name, value in sol_data.items():
                    data[col_name].append(value)
                sol_validity_checks = validity_checks.get(sol)
                if sol_validity_checks:
                    data['validity_checks'].append(sol_validity_checks)
                else:
                    data['validity_checks'].append({})
            
            table = pandas.DataFrame(data)
            return table







import aiohttp
import asyncio
import datetime

try:
    import pandas
except ImportError:
    pandas = None

from ..client import BaseClient
from ..errors import *


class Exoplanet(BaseClient):
    """
    Client for NASA Exoplanet Archive API.

    In-depth documentation for this API can be found at https://exoplanetarchive.ipac.caltech.edu/docs/program_interfaces.html

    Parameters for Exoplanet API:
        -
    """

    async def raw_get(self, query):
        pass

    async def get(self, table: str, select=None, count=None, colset=None, where=None, order=None, ra=None, dec=None, format=None):
        pass

